package com.sun.xml.internal.ws.encoding;

import java.awt.Graphics;
import java.awt.MediaTracker;
import javax.imageio.stream.ImageOutputStream;
import java.util.Iterator;
import javax.imageio.ImageWriter;
import java.awt.image.BufferedImage;
import java.io.OutputStream;
import java.io.BufferedInputStream;
import java.io.IOException;
import javax.activation.DataSource;
import java.util.Arrays;
import javax.activation.ActivationDataFlavor;
import java.awt.Image;
import javax.imageio.ImageIO;
import java.awt.datatransfer.DataFlavor;
import java.util.logging.Logger;
import javax.activation.DataContentHandler;
import java.awt.Component;

public class ImageDataContentHandler extends Component implements DataContentHandler
{
    private static final Logger log;
    private final DataFlavor[] flavor;
    
    public ImageDataContentHandler() {
        String[] mimeTypes = ImageIO.getReaderMIMETypes();
        this.flavor = new DataFlavor[mimeTypes.length];
        for (int i = 0; i < mimeTypes.length; ++i) {
            this.flavor[i] = new ActivationDataFlavor(Image.class, mimeTypes[i], "Image");
        }
    }
    
    public DataFlavor[] getTransferDataFlavors() {
        return (DataFlavor[])Arrays.copyOf(this.flavor, this.flavor.length);
    }
    
    public Object getTransferData(DataFlavor df, DataSource ds) throws IOException {
        DataFlavor[] flavor = this.flavor;
        int length = flavor.length;
        int i = 0;
        while (i < length) {
            DataFlavor aFlavor = flavor[i];
            if (aFlavor.equals(df)) {
                return this.getContent(ds);
            }
            ++i;
        }
        return null;
    }
    
    public Object getContent(DataSource ds) throws IOException {
        return ImageIO.read(new BufferedInputStream(ds.getInputStream()));
    }
    
    public void writeTo(Object obj, String type, OutputStream os) throws IOException {
        try {
            BufferedImage bufImage;
            if ((obj instanceof BufferedImage)) {
                bufImage = (BufferedImage)obj;
            }
            else if ((obj instanceof Image)) {
                bufImage = this.render((Image)obj);
            }
            else {
                throw new IOException("ImageDataContentHandler requires Image object, was given object of type " + obj.getClass().toString());
            }
            ImageWriter writer = null;
            Iterator<ImageWriter> i = ImageIO.getImageWritersByMIMEType(type);
            if (i.hasNext()) {
                writer = (ImageWriter)i.next();
            }
            if (writer != null) {
                ImageOutputStream stream = ImageIO.createImageOutputStream(os);
                writer.setOutput(stream);
                writer.write(bufImage);
                writer.dispose();
                stream.close();
            }
            else {
                throw new IOException("Unsupported mime type:" + type);
            }
        }
        catch (Exception e) {
            throw new IOException("Unable to encode the image to a stream " + e.getMessage());
        }
    }
    
    private BufferedImage render(Image img) throws InterruptedException {
        MediaTracker tracker = new MediaTracker(this);
        tracker.addImage(img, 0);
        tracker.waitForAll();
        BufferedImage bufImage = new BufferedImage(img.getWidth(null), img.getHeight(null), 1);
        Graphics g = bufImage.createGraphics();
        g.drawImage(img, 0, 0, null);
        g.dispose();
        return bufImage;
    }
    
    static {
        log = Logger.getLogger(ImageDataContentHandler.class.getName());
    }
}
